"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("isomorphic-fetch");
var NoOpStorageBackend_1 = require("./NoOpStorageBackend");
var LocalStorageBackend_1 = require("./LocalStorageBackend");
var CacheProxy_1 = require("./CacheProxy");
var constants_1 = require("./constants");
var HearthstoneJSON = (function () {
    function HearthstoneJSON(storage) {
        var _this = this;
        this.storagePrefix = "hsjson-";
        this.endpoint = "https://api.hearthstonejson.com/v1/";
        this.defaultLocale = "enUS";
        this.cached = null;
        this.fallback = null;
        this.createUrl = function (build, locale) {
            return _this.endpoint + build + "/" + locale + "/cards.json";
        };
        this.extractBuild = function (url) {
            var endpointExpression = new RegExp(_this.endpoint.replace(/[\/.]/g, "\\$&"));
            var pathExpression = /((\d+)|(latest))\/[a-zA-Z]+\/cards\.json/;
            var pattern = new RegExp("^" + endpointExpression.source + pathExpression.source + "$");
            var matches = pattern.exec(url);
            if (!matches) {
                throw new Error('No build found in url "' + url + '"');
            }
            return matches[1];
        };
        if (storage === null) {
            this.storage = new NoOpStorageBackend_1.default();
        }
        else if (typeof storage === "undefined") {
            this.storage = new CacheProxy_1.default(new LocalStorageBackend_1.default());
        }
        else if (storage) {
            this.storage = storage;
        }
    }
    HearthstoneJSON.prototype.get = function (build, locale) {
        var _this = this;
        if (build === "latest") {
            return this.getLatest(locale);
        }
        if (!locale) {
            locale = this.defaultLocale;
        }
        this.fallback = false;
        return this.getSpecificBuild(build, locale).catch(function () {
            _this.fallback = true;
            return _this.fetchLatestBuild(locale);
        });
    };
    HearthstoneJSON.prototype.getLatest = function (locale) {
        if (!locale) {
            locale = this.defaultLocale;
        }
        this.fallback = false;
        return this.fetchLatestBuild(locale);
    };
    HearthstoneJSON.prototype.getSpecificBuild = function (build, locale) {
        var _this = this;
        var key = this.generateKey(build, locale);
        var bypassCache = false;
        if (this.storage.has(key)) {
            var stored = this.storage.get(key);
            // verify format
            if (typeof stored === "object" &&
                typeof stored["revision"] === "number" &&
                Array.isArray(stored["cards"])) {
                if (stored["revision"] >= this.getRevision(build)) {
                    this.cached = true;
                    return Promise.resolve(stored["cards"]);
                }
            }
            // local version is not valid or outdated, do a full reload
            bypassCache = true;
        }
        this.cached = false;
        return this.fetchSpecificBuild(build, locale, bypassCache).catch(function (error) {
            // possibly invalid CORS header in cache
            return _this.fetchSpecificBuild(build, locale, true);
        });
    };
    HearthstoneJSON.prototype.fetchSpecificBuild = function (build, locale, bypassCache) {
        var _this = this;
        var headers = new Headers();
        headers.set("accept", "application/json; charset=utf-8");
        return fetch(this.createUrl(build, locale), {
            method: "GET",
            mode: "cors",
            cache: bypassCache ? "reload" : "default",
            headers: headers,
        })
            .then(function (response) {
            var statusCode = response.status;
            if (statusCode !== 200) {
                throw new Error("Expected status code 200, got " + statusCode);
            }
            return response.json();
        })
            .then(function (payload) {
            _this.store(build, locale, payload);
            return payload;
        });
    };
    HearthstoneJSON.prototype.fetchLatestBuild = function (locale) {
        var _this = this;
        return this.fetchLatestBuildNumber(locale).then(function (build) {
            return _this.getSpecificBuild(build, locale);
        });
    };
    HearthstoneJSON.prototype.fetchLatestBuildNumber = function (locale) {
        var _this = this;
        return fetch(this.createUrl("latest", locale), {
            method: "HEAD",
            mode: "cors",
            cache: "no-store",
        }).then(function (response) {
            // we expect to be redirected
            var statusCode = response.status;
            if (statusCode !== 200) {
                throw new Error("Expected status code 200, got " + statusCode);
            }
            // extract build number
            var build = _this.extractBuild(response.url);
            if (isNaN(+build)) {
                throw new Error("Expected numeric build number");
            }
            var buildNumber = +build;
            return buildNumber;
        });
    };
    HearthstoneJSON.prototype.store = function (buildNumber, locale, payload) {
        if (!payload.length) {
            // this doesn't look right - refuse to cache this
            return;
        }
        var key = this.generateKey(buildNumber, locale);
        this.storage.set(key, {
            revision: this.getRevision(buildNumber),
            cards: payload,
        });
    };
    HearthstoneJSON.prototype.generateKey = function (build, locale) {
        if (build === "latest") {
            throw new Error('Refusing to generate key for "latest" metadata');
        }
        return this.storagePrefix + build + "_" + locale;
    };
    HearthstoneJSON.prototype.getRevision = function (build) {
        var revision = 0;
        if (typeof constants_1.REVISIONS[build] === "number") {
            revision = constants_1.REVISIONS[build];
        }
        return revision;
    };
    return HearthstoneJSON;
}());
exports.default = HearthstoneJSON;
//# sourceMappingURL=HearthstoneJSON.js.map